/******************************************************************************
  Copyright (C), 2013, Texas Instrument.
 ******************************************************************************
  File Name     : K2_Board_Init.h
  Version       : Initial Draft
  Author        : Brighton Feng
  Created       : July 8, 2013
  Last Modified :
  Description   : KeyStone 2 Board baisc configuration
  History       :
  1.Date        : July 8, 2013
    Author      : Brighton Feng
    Modification: Created file
******************************************************************************/
#ifndef _K2_BOARD_INIT_H_
#define _K2_BOARD_INIT_H_
#include "K2_DDR_Init.h"
#include "K2_common.h"

/*----------------------parameters based on board----------------------------*/
#if defined(K2L_EVM)
    #error Please define parameters for K2L_EVM.

#elif defined(K2E_EVM)
    #error Please define parameters for K2E_EVM.

#elif defined(K2H_EVM) || defined(K2K_EVM)
/*Digital PLLs reference clock in MHz,
target speed= xxx_REF_CLK_MHZ/xxx_DIVIDER*xxx_multiplier*/
/*122.88/4*39= 1198.08MHz*/
#define MAIN_PLL_REF_CLK_MHZ    122.88  
#define MAIN_PLL_DIVISOR        4
#define MAIN_PLL_MULTIPLIER     39
/*122.88/24*205= 1049.6MHz. 
please note, there is addtional 1/3 divider between PASSPLL and PASS*/
#define PASS_PLL_REF_CLK_MHZ    122.88  
#define PASS_PLL_DIVISOR        24
#define PASS_PLL_MULTIPLIER     205
/*125/5*48=1200MHz*/
#define ARM_PLL_REF_CLK_MHZ     125     
#define ARM_PLL_DIVISOR         5
#define ARM_PLL_MULTIPLIER      48

/*Serdes reference clock speeed, must be set with CSL enum value*/
#define SRIOSGMII_SERDES_REF_CLK    CSL_SERDES_REF_CLOCK_156p25M
#define PCIE_SERDES_REF_CLK         CSL_SERDES_REF_CLOCK_100M
#define HYPERLINK_SERDES_REF_CLK    CSL_SERDES_REF_CLOCK_312p5M
#define AIF_SERDES_REF_CLK          CSL_SERDES_REF_CLOCK_122p88M
#define XFI_SERDES_REF_CLK          CSL_SERDES_REF_CLOCK_156p25M
#define USB_SERDES_REF_CLK          CSL_SERDES_REF_CLOCK_100M

/*DDR3 memory size in bytes*/
#define DDR3A_SIZE_BYTES    (1*1024*1024*1024UL)
#define DDR3B_SIZE_BYTES    (1*1024*1024*1024)

/*FLASH, EEPROM size in bytes*/
#define SPI_FLASH_SIZE_BYTES 	(16*1024*1024)
#define I2C_EEPROM_SIZE_BYTES 	(128*1024)

#else 
    #error Board type is NOT defined.
#endif

/*------------------------functions declaration----------------------------*/
/*configure DDR3A*/
extern void K2_DDR3A_config(DDR3_Arbitration_Config * arbitration_cfg, 
	DDR3_ECC_Config * ecc_cfg);

#if(2==CSL_DDR3_PER_CNT)
/*configure DDR3B*/
extern void K2_DDR3B_config(DDR3_Arbitration_Config * arbitration_cfg, 
	DDR3_ECC_Config * ecc_cfg);
#endif

#endif

